(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('d3-shape'), require('react-dom'), require('react'), require('react-roughjs'), require('d3-scale')) :
  typeof define === 'function' && define.amd ? define(['exports', 'd3-shape', 'react-dom', 'react', 'react-roughjs', 'd3-scale'], factory) :
  (factory((global.roughCharts = {}),global.d3Shape,global.ReactDOM,global.react,global.reactRoughjs,global.d3Scale));
}(this, (function (exports,d3Shape,ReactDOM,React,reactRoughjs,d3Scale) {
  var ChartContext = React.createContext(null);

  var isNil = function (x) { return x == null; };
  var isFunction = function (x) { return typeof x === 'function'; };
  var removeDuplicates = function (xs) {
    var res = {};
    xs.forEach(function (key) {
      res[key] = true;
    });
    return Object.keys(res);
  };
  var getBandWidth = function (scale) {
    var ticks = 'ticks' in scale ? scale.ticks() : scale.domain();

    if (ticks.length <= 1) {
      var ref = scale.range();
      var low = ref[0];
      var high = ref[1];
      return Math.abs(high - low);
    }

    return Math.abs(scale(ticks[1]) - scale(ticks[0]));
  };
  var processTooltipHandlers = function (child, handlers) {
    if (!React.isValidElement(child)) {
      return null;
    }

    var additionalProps = Object.entries(handlers).reduce(function (acc, ref) {
      var handlerName = ref[0];
      var f = ref[1];

      if (child.props[handlerName]) {
        acc[handlerName] = function (e) {
          f(e);
          child.props[handlerName](e);
        };
      } else {
        acc[handlerName] = f;
      }

      return acc;
    }, {});
    return React.cloneElement(child, additionalProps);
  };

  var getDomain = function (type, values) {
    var min = Math.min.apply(Math, values);
    return type === 'scaleBand' ? values : [min > 0 ? 0 : min, Math.max.apply(Math, values)];
  };

  function useChartContext(props, scaleKeyName) {
    var value = React.useContext(ChartContext);

    if (value === null) {
      throw Error('Chart Components must be wrapped inside <ChartProvider>!');
    }

    var data = value.data;
    var contentWidth = value.contentWidth;
    var contentHeight = value.contentHeight;
    var setScaleData = value.setScaleData;
    var scaleData = value.scaleData;
    var userXScale = scaleData.userXScale;
    var userYScale = scaleData.userYScale;
    var internalXScale = scaleData.internalXScale;
    var internalYScale = scaleData.internalYScale;
    var dataKey = props.dataKey;
    React.useEffect(function () {
      if (!scaleKeyName) {
        return;
      }

      if (scaleKeyName === 'xDataKey') {
        if (userXScale) {
          userXScale.range([0, contentWidth]);
          setScaleData(function (prev) { return (Object.assign({}, prev,
            {xScale: userXScale,
            xDataKey: dataKey})); });
          return;
        }

        var values = data.map(function (d) { return d[dataKey]; });
        internalXScale.range([0, contentWidth]);
        var domain = getDomain('scaleBand', values);
        internalXScale.domain(domain);
        setScaleData(function (prev) {
          var obj;

          return (Object.assign({}, prev,
          ( obj = {}, obj[scaleKeyName] = dataKey, obj ),
          {xScale: internalXScale}));
        });
      }

      if (scaleKeyName === 'yDataKey' && dataKey) {
        if (userYScale) {
          userYScale.range([0, contentHeight]);
          setScaleData(function (prev) { return (Object.assign({}, prev,
            {yScale: userYScale,
            yDataKey: dataKey})); });
        } else {
          var values$1 = data.map(function (d) { return d[dataKey]; });
          internalYScale.range([0, contentHeight]);
          var domain$1 = getDomain('scaleLinear', values$1);
          internalYScale.domain(domain$1);
          setScaleData(function (prev) {
            var obj;

            return (Object.assign({}, prev,
            ( obj = {}, obj[scaleKeyName] = dataKey, obj ),
            {yScale: internalYScale}));
          });
        }
      }

      var getSeriesScaleData = function (prev) {
        var obj;

        var barDataKeys = prev.barDataKeys;
        var lineDataKeys = prev.lineDataKeys;
        var yDataKey = prev.yDataKey;
        var circleDataKeys = prev.circleDataKeys;
        var prevDataKeys = prev[scaleKeyName];
        var newDataKeys = prevDataKeys.indexOf(dataKey) > -1 ? prevDataKeys : prevDataKeys.concat( [dataKey]);
        var newScaleData = Object.assign({}, prev,
          ( obj = {}, obj[scaleKeyName] = newDataKeys, obj ));
        var values = yDataKey ? data.map(function (d) { return d[yDataKey]; }) : removeDuplicates(newDataKeys.concat( lineDataKeys, barDataKeys, circleDataKeys)).reduce(function (acc, groupName) {
          acc.push.apply(acc, data.map(function (item) { return +item[groupName]; }));
          return acc;
        }, []); // TODO xScale

        if (userYScale) {
          userYScale.range([contentHeight, 0]);
          newScaleData.yScale = userYScale;
        } else {
          var domain = getDomain('scaleLinear', values);
          internalYScale.domain(domain);
          internalYScale.range([contentHeight, 0]);
          newScaleData.yScale = internalYScale;
        }

        return newScaleData;
      };

      if (scaleKeyName === 'barDataKeys') {
        setScaleData(function (prev) {
          var newScaleData = getSeriesScaleData(prev);
          var barScale = d3Scale.scaleBand().domain(newScaleData[scaleKeyName]);
          newScaleData.barScale = barScale;
          return newScaleData;
        });
      }

      if (scaleKeyName === 'lineDataKeys' || scaleKeyName === 'circleDataKeys') {
        setScaleData(function (prev) { return getSeriesScaleData(prev); });
      }
    }, [scaleKeyName, dataKey, contentHeight, contentWidth]);
    var margin = Object.assign({}, value.margin);
    var width = value.width;
    var height = value.height;
    return Object.assign({}, value,
      {width: width,
      height: height,
      contentWidth: width - margin.left - margin.right,
      contentHeight: height - margin.top - margin.bottom,
      options: Object.assign({}, value.options,
        (props.options || {})),
      margin: margin,
      data: !isNil(props.data) ? props.data : value.data,
      scaleData: value.scaleData,
      setScaleData: value.setScaleData});
  }

  /* eslint-disable react/no-array-index-key */
  var XAxis = function (props) {
    if (!props.dataKey) {
      throw Error('dataKey of XAxisProps is required');
    }

    var ref = useChartContext(props, 'xDataKey');
    var options = ref.options;
    var contentHeight = ref.contentHeight;
    var contentWidth = ref.contentWidth;
    var scaleData = ref.scaleData;
    var xScale = scaleData.xScale;
    var tickSize = props.tickSize;
    var fontSize = props.fontSize;
    var format = props.format;
    var tickCount = props.tickCount;

    if (!xScale) {
      return null;
    }

    var ticks = 'ticks' in xScale ? xScale.ticks(tickCount) : xScale.domain();
    var bandwidth = getBandWidth(xScale);
    var y = contentHeight;
    return React.createElement(React.Fragment, null, React.createElement(reactRoughjs.Line, {
      x1: 0,
      y1: y,
      x2: contentWidth + bandwidth / 2,
      y2: y,
      options: Object.assign({}, {bowing: 0.2},
        options)
    }), ticks.map(function (t, index) { return React.createElement(React.Fragment, {
      key: index
    }, React.createElement(reactRoughjs.Line, {
      x1: xScale(t) + bandwidth / 2,
      y1: y,
      x2: xScale(t) + bandwidth / 2,
      y2: y + tickSize,
      options: Object.assign({}, {strokeWidth: 2},
        options)
    }), React.createElement("text", {
      x: xScale(t) + bandwidth / 2,
      y: y + tickSize + fontSize,
      stroke: options.stroke,
      fill: options.stroke,
      textAnchor: "middle"
    }, isFunction(format) ? format(t) : String(t))); }));
  };
  XAxis.displayName = 'XAxis';
  XAxis.defaultProps = {
    tickSize: 10,
    fontSize: 16,
    tickCount: 10
  };

  /* eslint-disable react/no-array-index-key */
  var YAxis = function (props) {
    var tickSize = props.tickSize;
    var fontSize = props.fontSize;
    var format = props.format;
    var tickCount = props.tickCount;
    var ref = useChartContext(props, 'yDataKey');
    var options = ref.options;
    var contentHeight = ref.contentHeight;
    var scaleData = ref.scaleData;
    var scale = scaleData.yScale;

    if (!scale) {
      return null;
    }

    var ticks = scale.ticks ? scale.ticks(tickCount) : scale.domain();
    return React.createElement(React.Fragment, null, React.createElement(reactRoughjs.Line, {
      x1: 0,
      y1: 0,
      x2: 0,
      y2: contentHeight,
      options: Object.assign({}, {bowing: 0.2},
        options)
    }), ticks.map(function (t, index) { return React.createElement(React.Fragment, {
      key: index
    }, React.createElement(reactRoughjs.Line, {
      x1: 0 - tickSize,
      x2: 0,
      y1: scale(t),
      y2: scale(t),
      options: Object.assign({}, {strokeWidth: 2},
        options)
    }), React.createElement("text", {
      x: 0 - tickSize,
      y: scale(t) + fontSize / 3,
      stroke: options.stroke,
      fill: options.stroke,
      textAnchor: "end"
    }, isFunction(format) ? format(t) : String(t))); }));
  };
  YAxis.displayName = 'YAxis';
  YAxis.defaultProps = {
    tickSize: 10,
    fontSize: 16,
    tickCount: 10
  };

  var mousePositions = function (event) {
    var IE = !!document.all;
    var x;
    var y;

    if (IE) {
      x = event.clientX + document.body.scrollLeft;
      y = event.clientY + document.body.scrollTop;
    } else {
      x = window.Event ? event.pageX : event.clientX + (document.documentElement.scrollLeft ? document.documentElement.scrollLeft : document.body.scrollLeft);
      y = window.Event ? event.pageY : event.clientY + (document.documentElement.scrollTop ? document.documentElement.scrollTop : document.body.scrollTop);
    }

    return {
      x: x,
      y: y
    };
  };

  var useTooltipGenerator = function (props) {
    var dataKey = props.dataKey;

    if (!dataKey) {
      throw Error('dataKey is Required!');
    }

    var ref = useChartContext(props);
    var setTooltipData = ref.setTooltipData;

    var generateHandlers = function (item, content) {
      if ( content === void 0 ) content = {
      name: '',
      value: ''
    };

      return ({
      onMouseOver: function (e) {
        var ref = mousePositions(e);
        var x = ref.x;
        var y = ref.y;
        setTooltipData(function (prev) { return (Object.assign({}, prev,
          {showToolTip: true,
          x: x,
          y: y,
          name: content.name,
          value: content.value,
          activeItem: item})); });
      },
      onMouseMove: function (e) {
        var ref = mousePositions(e);
        var x = ref.x;
        var y = ref.y;
        setTooltipData(function (prev) { return (Object.assign({}, prev,
          {showToolTip: true,
          x: x,
          y: y,
          activeItem: item})); });
      },
      onMouseOut: function () {
        setTooltipData(function (prev) { return (Object.assign({}, prev,
          {showToolTip: false})); });
      }
    });
    };

    return {
      generateHandlers: generateHandlers
    };
  };

  /* eslint-disable react/no-array-index-key */
  var BarSeries = function (props) {
    var dataKey = props.dataKey;
    var options = props.options;
    var ref = useChartContext(props, 'barDataKeys');
    var data = ref.data;
    var contentHeight = ref.contentHeight;
    var scaleData = ref.scaleData;
    var xScale = scaleData.xScale;
    var barScale = scaleData.barScale;
    var yScale = scaleData.yScale;
    var xDataKey = scaleData.xDataKey;
    var ref$1 = useTooltipGenerator(props);
    var generateHandlers = ref$1.generateHandlers;

    if (!dataKey) {
      throw Error('dataKey is Required!');
    } // TODO xDataKey


    if (!xScale || !barScale || !yScale || !xDataKey) {
      return null;
    }

    var width = getBandWidth(xScale);
    var offset = width * 0.2 / 2;

    var generateChildProps = function (item) {
      var x = xScale(item[xDataKey]);
      barScale.range([x + offset, x + width - offset]);
      var min = yScale.domain()[0];
      var y0 = yScale(0);
      var itemY = yScale(item[dataKey]);
      var y = min >= 0 ? itemY : yScale(Math.max(item[dataKey], 0));
      var height = min >= 0 ? contentHeight - itemY : Math.abs(y0 - itemY);
      return {
        x: barScale(dataKey),
        y: y,
        width: barScale.bandwidth(),
        height: height,
        options: Object.assign({}, {fill: 'black'},
          options)
      };
    };

    var children = props.children;
    return React.createElement(React.Fragment, null, data.map(function (item, index) {
      var handlers = generateHandlers(item, {
        name: ("" + dataKey),
        value: ("" + (item[dataKey]))
      });
      var childProps = generateChildProps(item);
      return isFunction(children) ? processTooltipHandlers(children(item, childProps, index), handlers) : React.createElement(reactRoughjs.Rectangle, Object.assign({
        key: index
      }, childProps, handlers));
    }));
  };
  BarSeries.displayName = 'BarSeries';

  function objectWithoutProperties (obj, exclude) { var target = {}; for (var k in obj) if (Object.prototype.hasOwnProperty.call(obj, k) && exclude.indexOf(k) === -1) target[k] = obj[k]; return target; }
  var Arc = function (props) {
    var innerRadius = props.innerRadius;
    var outerRadius = props.outerRadius;
    var startAngle = props.startAngle;
    var endAngle = props.endAngle;
    var padAngle = props.padAngle;
    var options = props.options;
    var cx = props.cx;
    var cy = props.cy;
    var rest$1 = objectWithoutProperties( props, ["innerRadius", "outerRadius", "startAngle", "endAngle", "padAngle", "options", "cx", "cy"] );
    var rest = rest$1;
    var a = d3Shape.arc();
    var d = a({
      innerRadius: innerRadius,
      outerRadius: outerRadius,
      startAngle: startAngle,
      endAngle: endAngle,
      padAngle: padAngle
    });
    return React.createElement(reactRoughjs.Path, Object.assign({
      d: d,
      transform: ("translate(" + cx + ", " + cy + ")"),
      options: options
    }, rest));
  };
  Arc.displayName = 'RcArc';
  Arc.defaultProps = {
    innerRadius: 0,
    outerRadius: 0,
    startAngle: 0
  };

  /* eslint-disable react/no-array-index-key */
  var ArcSeries = function (props) {
    var ref = useChartContext(props);
    var data = ref.data;
    var options = ref.options;
    var contentHeight = ref.contentHeight;
    var contentWidth = ref.contentWidth;
    var ref$1 = useTooltipGenerator(props);
    var generateHandlers = ref$1.generateHandlers;
    var dataKey = props.dataKey;
    var cx = props.cx;
    var cy = props.cy;
    var innerRadiusPercent = props.innerRadiusPercent;
    var outerRadiusPercent = props.outerRadiusPercent;
    var padAngleProp = props.padAngle;
    var startAngleProp = props.startAngle;
    var endAngleProp = props.endAngle;
    var children = props.children;
    var newCx = !isNil(cx) ? cx : contentWidth / 2;
    var newCy = !isNil(cy) ? cy : contentHeight / 2;
    var radius = Math.min(newCx, newCy);
    var arcs = d3Shape.pie().startAngle(startAngleProp).padAngle(padAngleProp).endAngle(endAngleProp)(data.map(function (d) { return d[dataKey]; }));
    var arcsData = arcs.map(function (arc) {
      var startAngle = arc.startAngle;
      var endAngle = arc.endAngle;
      var padAngle = arc.padAngle;
      return {
        innerRadius: radius * innerRadiusPercent,
        outerRadius: radius * outerRadiusPercent,
        startAngle: startAngle,
        endAngle: endAngle,
        padAngle: padAngle
      };
    });
    var total = data.reduce(function (acc, d) { return acc + d[dataKey]; }, 0);
    return React.createElement(React.Fragment, null, arcsData.map(function (arcProps, index) {
      var item = data[index];
      var percent = (item[dataKey] / total * 100).toPrecision(3);
      var handlers = generateHandlers(item, {
        name: (dataKey + " " + (item[dataKey])),
        value: (percent + "%")
      });
      var childProps = Object.assign({}, arcProps,
        {cx: newCx,
        cy: newCy,
        options: Object.assign({}, {fill: 'black'},
          options)});

      if (isFunction(children)) {
        return processTooltipHandlers(children(item, childProps, index), handlers);
      }

      return React.createElement(Arc, Object.assign({
        key: index
      }, childProps, handlers));
    }));
  };
  ArcSeries.displayName = 'ArcSeries';
  ArcSeries.defaultProps = {
    innerRadiusPercent: 0,
    outerRadiusPercent: 1,
    padAngle: 0,
    startAngle: 0,
    endAngle: Math.PI * 2
  };

  var Y0 = function (props) {
    var ref = useChartContext(props);
    var yScale = ref.scaleData.yScale;
    var contentWidth = ref.contentWidth;

    if (!yScale) {
      return null;
    }

    return React.createElement(reactRoughjs.Line, {
      y1: yScale(0),
      y2: yScale(0),
      x1: 0,
      x2: contentWidth,
      strokeDasharray: "30",
      options: Object.assign({}, {bowing: 0.2},
        props.options)
    });
  };
  Y0.displayName = 'Y0';

  /* eslint-disable react/no-array-index-key */
  var CircleSeries = function (props) {
    var ref = useChartContext(props, 'circleDataKeys');
    var scaleData = ref.scaleData;
    var data = ref.data;
    var options = ref.options;
    var xScale = scaleData.xScale;
    var yScale = scaleData.yScale;
    var xDataKey = scaleData.xDataKey;
    var dataKey = props.dataKey;
    var ref$1 = useTooltipGenerator(props);
    var generateHandlers = ref$1.generateHandlers;

    if (!xScale || !yScale || !dataKey || !xDataKey) {
      return null;
    }

    var bandwidth = getBandWidth(xScale);
    var points = data.map(function (item) { return [xScale(item[xDataKey]), yScale(item[dataKey])]; });
    var children = props.children;
    return React.createElement(React.Fragment, null, points.map(function (ref, index) {
      var x = ref[0];
      var y = ref[1];

      var childProps = {
        x: x + bandwidth / 2,
        y: y,
        diameter: 10,
        options: Object.assign({}, options)
      };
      var item = data[index];
      var handlers = generateHandlers(data[index], {
        name: (dataKey + " " + (item[xDataKey])),
        value: ("" + (item[dataKey]))
      });

      if (isFunction(children)) {
        return processTooltipHandlers(children(data[index], childProps, index), handlers);
      }

      return React.createElement(reactRoughjs.Circle, Object.assign({
        key: index
      }, childProps, handlers));
    }));
  };
  CircleSeries.displayName = 'CircleSeries';

  /* eslint-disable react/no-array-index-key */
  var LineSeries = function (props) {
    var ref = useChartContext(props, 'lineDataKeys');
    var scaleData = ref.scaleData;
    var data = ref.data;
    var options = ref.options;
    var xScale = scaleData.xScale;
    var yScale = scaleData.yScale;
    var xDataKey = scaleData.xDataKey;
    var dataKey = props.dataKey;
    var curve = props.curve;

    if (!xScale || !yScale || !dataKey || !xDataKey) {
      return null;
    }

    var bandwidth = getBandWidth(xScale);
    var points = data.map(function (item) { return [xScale(item[xDataKey]), yScale(item[dataKey])]; });
    var line = d3Shape.line().curve(curve);
    var path = line(points);
    return React.createElement(React.Fragment, null, React.createElement(reactRoughjs.Path, {
      transform: ("translate(" + (bandwidth / 2) + ", 0)"),
      d: path,
      options: options
    }), React.createElement(CircleSeries, Object.assign({}, props)));
  };
  LineSeries.displayName = 'LineSeries';
  LineSeries.defaultProps = {
    curve: d3Shape.curveCardinal.tension(0.5)
  };

  var Tooltip = function (props) {
    var ref = useChartContext(props);
    var tooltipData = ref.tooltipData;
    var setTooltipData = ref.setTooltipData;
    var contentWidth = ref.contentWidth;
    var options = ref.options;
    var x = tooltipData.x;
    var y = tooltipData.y;
    var showToolTip = tooltipData.showToolTip;
    var name = tooltipData.name;
    var value = tooltipData.value;
    var activeItem = tooltipData.activeItem;
    var propsWidth = props.width;
    var height = props.height;
    var fontSize = props.fontSize;
    var children = props.children;
    var ref$1 = React.useState(120);
    var internalWidth = ref$1[0];
    var setWidth = ref$1[1];
    var textRef = React.useRef();
    var width = isNil(propsWidth) ? internalWidth : propsWidth;
    React.useEffect(function () {
      var node = textRef.current;

      if (node) {
        var ref = node.getBBox();
        var textWidth = ref.width;
        setWidth(textWidth + 30);
      }
    }, [textRef.current]);

    if (!showToolTip || !activeItem) {
      return null;
    } // TODO pintX pointY


    var left = contentWidth - x < width + 20 ? x - 20 - width : x + 20;

    var ellipse = function (s, length) {
      if ( s === void 0 ) s = '';

      return s.length > length ? ((s.slice(0, length)) + "...") : s;
    };

    var renderChildren = function () {
      var text = function (s) { return React.createElement("text", {
        ref: textRef,
        textAnchor: "middle",
        stroke: "black",
        fill: "black",
        x: width / 2,
        y: height / 2 + fontSize / 3
      }, s); };

      if (isFunction(children)) {
        var content = children(activeItem);
        return typeof content === 'string' ? text(content) : content;
      }

      return text(((ellipse(name, 20)) + ": " + (ellipse(value, 5))));
    };

    var content = React.createElement("div", {
      style: {
        position: 'absolute',
        left: left,
        top: y - height / 2
      }
    }, React.createElement("svg", {
      width: width + 4,
      height: height + 4
    }, React.createElement("g", {
      transform: "translate(2, 2)"
    }, React.createElement(reactRoughjs.RoughProvider, null, React.createElement(reactRoughjs.Rectangle, {
      width: width,
      height: height,
      onMouseOver: function (e) {
        setTooltipData(function (prev) { return (Object.assign({}, prev,
          {x: e.clientX,
          y: e.clientY})); });
      },
      onMouseMove: function (e) {
        setTooltipData(function (prev) { return (Object.assign({}, prev,
          {x: e.clientX,
          y: e.clientY})); });
      },
      onMouseOut: function () {
        setTooltipData(function (prev) { return (Object.assign({}, prev,
          {showToolTip: false})); });
      },
      options: Object.assign({}, {fill: 'white',
        fillStyle: 'solid',
        strokeWidth: 2},
        options)
    })), React.createElement(reactRoughjs.RoughProvider, null, renderChildren()))));
    return ReactDOM.createPortal(content, document.body);
  };
  Tooltip.displayName = 'Tooltip';
  Tooltip.defaultProps = {
    height: 40,
    fontSize: 16
  };

  var Provider = ChartContext.Provider;
  var defaultMargin = {
    top: 10,
    right: 10,
    bottom: 50,
    left: 50
  };
  var ChartProvider = function (props) {
    var ref$1 = React.useState(0);
    var innerHeight = ref$1[0];
    var setInnerHeight = ref$1[1];
    var ref$2 = React.useState(0);
    var innerWidth = ref$2[0];
    var setInnerWidth = ref$2[1];
    var ref = React.useRef();
    var internalXScale = d3Scale.scaleBand();
    var internalYScale = d3Scale.scaleLinear();
    var ref$3 = React.useState({
      barDataKeys: [],
      lineDataKeys: [],
      circleDataKeys: [],
      xScale: props.xScale || internalXScale,
      yScale: props.yScale || internalYScale,
      userXScale: props.xScale,
      userYScale: props.yScale,
      internalXScale: internalXScale,
      internalYScale: internalYScale
    });
    var scaleData = ref$3[0];
    var setScaleData = ref$3[1];
    var ref$4 = React.useState({
      x: -1,
      y: -1,
      showToolTip: false,
      name: '',
      value: '',
      activeItem: null
    });
    var tooltipData = ref$4[0];
    var setTooltipData = ref$4[1];
    React.useEffect(function () {
      var handleResize = function () {
        if (ref.current && (isNil(props.height) || isNil(props.width))) {
          var boundingRect = ref.current.parentElement.getBoundingClientRect();
          setInnerHeight(boundingRect.height);
          setInnerWidth(boundingRect.width);
        }
      };

      handleResize();
      window.addEventListener('resize', handleResize);
      return function () { return window.removeEventListener('resize', handleResize); };
    }, [props.height, props.width, ref.current]);
    var height = !isNil(props.height) ? props.height : innerHeight;
    var width = !isNil(props.width) ? props.width : innerWidth;
    var propsMargin = props.margin;
    var margin = Object.assign({}, defaultMargin,
      propsMargin);

    var shouldRenderChildren = function () { return height && width; };

    return React.createElement(Provider, {
      value: Object.assign({}, props,
        {height: height,
        width: width,
        margin: margin,
        scaleData: scaleData,
        setScaleData: function (f) { return setScaleData(f); },
        contentHeight: height - margin.bottom - margin.top,
        contentWidth: width - margin.left - margin.right,
        tooltipData: tooltipData,
        setTooltipData: function (f) { return setTooltipData(f); }})
    }, React.createElement("svg", {
      height: height,
      width: width,
      ref: ref
    }, React.createElement(reactRoughjs.RoughProvider, {
      transform: ("translate(" + (margin.left) + ", " + (margin.top) + ")"),
      config: {
        options: props.options
      }
    }, shouldRenderChildren() && props.children)));
  };
  ChartProvider.displayName = 'ChartProvider';
  ChartProvider.defaultProps = {
    data: [],
    options: {},
    margin: defaultMargin
  };

  exports.Rectangle = reactRoughjs.Rectangle;
  exports.Circle = reactRoughjs.Circle;
  exports.XAxis = XAxis;
  exports.YAxis = YAxis;
  exports.BarSeries = BarSeries;
  exports.ArcSeries = ArcSeries;
  exports.Arc = Arc;
  exports.Y0 = Y0;
  exports.LineSeries = LineSeries;
  exports.CircleSeries = CircleSeries;
  exports.Tooltip = Tooltip;
  exports.ChartProvider = ChartProvider;

})));
//# sourceMappingURL=index.umd.js.map
